#ifndef ARRAY_H
#define ARRAY_H
#include <iostream.h>
#include <string.h>

int zeros_int[256];
float zeros_float[256];
bool array_print=true;

void arrays(void) {
	for (int c1=0;c1<256;c1++) {zeros_int[c1]=0; zeros_float[c1]=0;
	};
};

template <class type> class array {
	public:
		array<type>(void);
		virtual ~array<type>(void);					//virtual destructor (for matrix.h too)
		array<type>(int n,int *s,type f);		//ndims,size
		array<type>(const array<type>& a);	//cpy constructor
		array<type>(char *fn);							//read from file
		void store(char *fn);								//write to file
		type token(void);										//periodic tokenizer (loops forever, if you let him)
		type token(int d);									//tokenizer -1 | 0 | 1 (actually takes any offset! non periodic!)
		type cycle(int d);									//sad, that we had to indroduce this function
		void record(type a);								//injects type at curr++ (abuses size[0] for current)
		array<int> sort(void);							//sorts array	and returns indices such that ???
		array<int> find(void);
		array<int> find(int n);
		void operator= (array<type>& a);    //equal		
		void operator= (type a);    				//equal		
		void apply(type (*func)(type));
		void apply(double (*func)(double));
		bool all(void);
		int ndims,numel;
		type *data;
		int *size; //size[0..ndims-1] is actual size | size[ndims] indexes current element
		//type operator() (const int& a,const int& b,const int& c);
		type operator()(int a) {return data[a];};
		type* operator[](int a) {return &data[a];};		
	private:
		void qsort(int a,int b,int *ind);
};
//constructors-------------------------------------------------------------
template <class type> array<type>::array(void) {data=NULL; size=NULL;};
template <class type> array<type>::~array(void) {
	if (data) {delete[] data;	delete[] size;}	
};
template <class type> array<type>::array(int n,int *s,type f) {
	ndims=n;
	size=new int[ndims+1];
	memcpy(size,s,4*ndims);	size[ndims]=0; //tokenizer prep
	numel=1; for (int c1=0;c1<ndims;c1++) numel*=size[c1];	
	data=new type[numel];	
	for (c1=0;c1<numel;c1++) data[c1]=f;
};
template <class type> array<type>::array(const array<type>& a) {
	ndims=a.ndims; numel=a.numel;
	size=new int[ndims+1];
	data=new type[numel];
	memcpy(size,a.size,4*(ndims+1));
	memcpy(data,a.data,sizeof(type)*numel);	
};
//file i/o------------------------------------------------------------------
template <class type> array<type>::array(char *fn) {//constructor!!!
	FILE *fid; //format: ndims,size[0:ndims-1],data
	if (fid=fopen(fn,"rb")) {
		fread(&ndims,4,1,fid);
		size=new int[ndims+1];
		fread(size,4,ndims,fid); size[ndims]=0; //tokenizer prep
		numel=1; for (int c1=0;c1<ndims;c1++) numel*=size[c1];		
		data=new type[numel];
		fread(data,sizeof(type),numel,fid);
		fclose(fid);
		cout << fn << " ->\t" << *this;
	} else {
		data=NULL;
		cout << "??? " << fn << " input suppressed!" << endl;
	}
};
template <class type> void array<type>::store(char *fn) {
	FILE *fid; //format: ndims,size[0:ndims-1],data
	if (fid=fopen(fn,"wb")) {
		fwrite(&ndims,4,1,fid);
		fwrite(size,4,ndims,fid);
		fwrite(data,sizeof(type),numel,fid);
		fclose(fid);
		cout << fn << " <-\t" << *this;
	} else cout << "??? " << fn << " output suppressed!" << endl;
}
//soft io funcs-------------------------------------------------------------
template <class type> type array<type>::token(void) {
	if (size[ndims]<numel) return data[size[ndims]++];
		else {size[ndims]=1; return data[0];}
};
template <class type> type array<type>::token(int d) {//change!!!
				if (size[ndims]>=numel) size[ndims]=numel-1;
	else 	if (size[ndims] <0)     size[ndims]=0;
	type ret=data[size[ndims]]; 	size[ndims]+=d;
	return ret;
};
template <class type> type array<type>::cycle(int d) {	//fundamental change!!!!
	type ret=data[size[ndims]]; 	size[ndims]+=d;         //size[ndims] has curtoken address
				if (size[ndims]>=numel) size[ndims]=0;
	else 	if (size[ndims] <0)     size[ndims]=numel-1;
	return ret;
};
template <class type> void array<type>::record(type a) {
	if (size[ndims]<numel) data[size[ndims]++]=a;
		else {size[ndims]=1; data[0]=a;}
};
//member funcs-------------------------------------------------------------
template <class type> void array<type>::apply(type (*func)(type)) {	
	for (int c1=0;c1<numel;c1++) data[c1]=func(data[c1]);
};	
template <class type> void array<type>::apply(double (*func)(double)) {	
	for (int c1=0;c1<numel;c1++) data[c1]=func((double)data[c1]);
};

//advanced member funcs-------------------------------------------------------------
template <class type> array<int> array<type>::sort(void) {	
	array<int> ind(1,size,-1);
	for (int c1=0;c1<size[0];c1++) ind.data[c1]=c1;
	qsort(0,size[0]-1,ind.data);
	return ind;
};
template <class type> void array<type>::qsort(int a,int b,int *ind) {
	int c1=a,c2=b,tpos;
	type ref=data[a],tdat; //selects first element!!!
	do {
		while (data[c1]<ref) c1++;
		while (data[c2]>ref) c2--;
		if (c1<=c2) {
					tpos=ind[c1];
			 ind[c1]=ind[c2];
			 ind[c2]=tpos;
					tdat=data[c1];
			data[c1]=data[c2];
			data[c2]=tdat;
			c1++;	c2--;
		}
	} while (c1<=c2);
	if (c2>a) qsort(a,c2,ind);
	if (c1<b) qsort(c1,b,ind);
};
template <class type> array<int> array<type>::find(void) {
	int cnt=0;
	for (int c1=0;c1<numel;c1++) if (data[c1]) cnt++;
	return find(cnt);
};
template <class type> array<int> array<type>::find(int n) {
	array<int> ind(1,&n,0);
	int cnt=0;
	for (int c1=0;c1<numel;c1++) 
		if (data[c1]&&cnt<n) ind.data[cnt++]=c1;
	return ind;
};
//operators----------------------------------------------------------------
template <class type> void array<type>::operator= (array<type>& a) {					
	if (data==a.data) return;
	if (data) {delete[] data;	delete[] size;}
	ndims=a.ndims; size=new int[ndims+1];
	numel=a.numel; data=new type[numel];
	memcpy(size,a.size,4*(ndims+1));
	memcpy(data,a.data,sizeof(type)*numel);
};
template <class type> void array<type>::operator= (type a) {
	for (int c1=0;c1<numel;c1++) data[c1]=a;	
};

template <class type> bool array<type>::all(void) {		
	for (int c1=0;c1<numel;c1++)
		if (!data[c1]) return false;
	return true;
};
template <class type> ostream& operator<< (ostream& os, array<type>& a);
template <class type> ostream& operator<< (ostream& os, array<type>& a) {		
	os << " [";
	if (!a.data) os << "empty]" << endl;
	else {
		for (int c1=0;c1<a.ndims-1;c1++) os << a.size[c1] << "x";
		os << a.size[a.ndims-1] << "]=" << a.numel << "\tdata:" << a.data << endl;
		if (array_print) {//global state
			if (a.ndims==1)
				if (a.numel<=100) {
					for (int c1=0;c1<a.size[0];c1++) os << a.data[c1] << "\t";
					os << endl;
				}
			if (a.ndims==2)
				if (a.size[0]<=15&&a.numel<=100)
					for (int c1=0;c1<a.size[0];c1++) {
						for (int c2=0;c2<a.size[1];c2++) os << a.data[c1+c2*a.size[0]] << "\t";
						os << endl;
					}
		}
	}
	return os;
};

//template <class type> type array<type>::operator() (const int& a,const int& b,const int& c) {
//	//return a+b*size[0]+c*size[0]*size[1];
//	return data[a+b*size[0]+c*size[0]*size[1]];
//};	


/*template <class type> istream& operator>> (istream& is, array<type>& a);
template <class type> istream& operator>> (istream& is, array<type>& a) {
	char temp[MAXLEN];
	//NOT WORKING!!!
	is.getline(temp,MAXLEN);
	//a.cpy(temp); 
	return is; };
*/
#endif