#ifndef HASH_H
#define HASH_H
#include <iostream.h>
#include <string.h>

template <class comp,class type> class bucket {
	public:
		comp key;
		type data;
};

//class comp has to have an == operator to it
template <class comp,class type> class hash {
	public:
		hash<comp,type>(void);
		~hash<comp,type>(void);
		hash<comp,type>(int n,unsigned int (*f)(comp));		//init with # of buckets and hash function (ptr)
		bool find(comp key,type& data);										//finds element by key and returns data byref otherwise false
		bool store(comp key,type data);										//stores object with key if key does not exist already
		bool remove(comp key);														//removes existing entry with key otherwise false (alternatively make it return data!!!)
		bool apply(bool (*f)(type));
		bool apply(bool (*f)(comp&,type));
		array<linked<bucket<comp,type> >*> *table;
		matrix<int> toMatrix(matrix<type> a);
		int howmany(void);
		int numel;
	private:
		unsigned int (*func)(comp);
};
//constructors-------------------------------------------------------------
template <class comp,class type> hash<comp,type>::hash(void) {table=NULL; numel=0;};
template <class comp,class type> hash<comp,type>::~hash(void) {
	if (table) {
		for (int c1=0;c1<table->numel;c1++) delete table->data[c1];
		delete table;
	};
};
template <class comp,class type> hash<comp,type>::hash(int n,unsigned int (*f)(comp)) {
	table=new array<linked<bucket<comp,type> >*>(1,&n,NULL);
	for (int c1=0;c1<table->numel;c1++)
		table->data[c1]=new linked<bucket<comp,type> >();
	func=f;
	numel=0;
};
template <class comp,class type> bool hash<comp,type>::find(comp key,type& data) {
	unsigned int n=func(key)%table->numel;
	bucket<comp,type> b;
	while (table->data[n]->token(b))
		if (b.key==key) {
			table->data[n]->reset();
			data=b.data;
			return true;
		}
	return false;
};
template <class comp,class type> bool hash<comp,type>::store(comp key,type data) {
	unsigned int n=func(key)%table->numel;
	bucket<comp,type> b;
	while (table->data[n]->token(b))
		if (b.key==key) {
			table->data[n]->reset();
			return false;
		}
	b.key=key; b.data=data;
	table->data[n]->append(b);
	numel++;
	return true;
};
template <class comp,class type> bool hash<comp,type>::remove(comp key) {
	unsigned int n=func(key)%table->numel;
	bucket<comp,type> b;
	while (table->data[n]->token(b))
		if (b.key==key) {			
			table->data[n]->remove();
			table->data[n]->reset();
			numel--;
			return true;
		}
	return false;
};//LATER: Merge find and remove and have find(HASH_FIND/HASH_REMOVE,...);

//template <class comp,class type> matrix<int> hash<comp,type>::toMatrix(int n,matrix<type> a) {
//	int cnt=0;
//	bucket<comp,type> b;
//	matrix<int> c(n,numel);
//	matrix<type> a(n,numel);
//	for (int c1=0;c1<table->numel;c1++)
//		while (table->data[c1]->token(b)) {
//			memcpy(c.data[n*(cnt++)]b.key.data,b.key.numel*4);
//		}
//
//};

template <class comp,class type> bool hash<comp,type>::apply(bool (*f)(type)) {	
	bucket<comp,type> b;
	for (int c1=0;c1<table->numel;c1++)
		while (table->data[c1]->token(b)) 
			if (!f(b.data)) {
				table->data[c1]->reset();
				return false;
			}
	return true;
};

template <class comp,class type> int hash<comp,type>::howmany(void) {
	int cnt=0;
	bucket<comp,type> b;
	for (int c1=0;c1<table->numel;c1++)
		while (table->data[c1]->token(b)) cnt++;
	return cnt;
}
template <class comp,class type> bool hash<comp,type>::apply(bool (*f)(comp&,type)) {
	bucket<comp,type> b;
	for (int c1=0;c1<table->numel;c1++)
		while (table->data[c1]->token(b)) 
			if (!f(b.key,b.data)) {
				table->data[c1]->reset();
				return false;
			}
	return true;
};
//---------------------------------------------------------------------------
template <class comp,class type> ostream& operator<< (ostream& os,hash<comp,type>& h) {
	bucket<comp,type> b;
	os << SEPR;
	for (int c1=0;c1<h.table->numel;c1++) {
		os << "{@";		
		while (h.table->data[c1]->token(b)) os << ",[" << b.key << "," << b.data << "]";
		os << "}" << endl;
	};
	return os;
};


//unsigned int hash_sparse_void(unsigned int a) {return a;};
//template <class type> class sparse : public hash<unsigned int,type> {
//	public:
//		sparse(){};
//		sparse(matrix<type> &a);
//		matrix<type> multMatSparse(matrix<type> &a);
//		~sparse() {};
//};
//
//template <class type> sparse<type>::sparse(matrix<type> &a) : hash<unsigned int,type>(a.size[1],hash_sparse_void) {
//	int c1,c2,ofs=0;	
//	bucket<unsigned int,type> b;
//	for (c1=0;c1<a.size[1];c1++) {
//		for (c2=0;c2<a.size[0];c2++)
//			if (a.data[ofs+c2]) {				
//				b.key=c2;
//				b.data=a.data[ofs+c2];
//				table->data[c1]->append(b);
//			}
//		ofs+=a.size[0];
//	}
//};
//
//template <class type> matrix<type> sparse<type>::multMatSparse(matrix<type> &a) {
//	int c1,c2,c3,ofs=0,of2=0;
//	matrix<float> x(a.size[0],table->numel,"");	
//	bucket<unsigned int,type> b;
//	for (c1=0;c1<table->numel;c1++) {
//		ofs=c1*a.size[0];
//		while (table->data[c1]->token(b)) {
//			of2=b.key*a.size[0];
//			for (c2=0;c2<a.size[0];c2++)
//				x.data[c2+ofs]+=b.data*a.data[c2+of2];
//		}
//	}	
//	return x;
//};


template <class type> class sparse {
	public:
		sparse(void) {ind=NULL; val=NULL; };
		~sparse(void);
		sparse(matrix<type> &a);
		matrix<type> full(void);
		matrix<type> multMatSparse(matrix<type> &a);
		matrix<int> *ind;
		array<type> *val;
		int size[2];
};

template <class type> sparse<type>::~sparse(void) {
	if (ind) {delete ind; delete val;}
};

template <class type> sparse<type>::sparse(matrix<type> &a) {
	int c1,c2,ofs=0;
	array<int> ptr=a.find();
	ind=new matrix<int>(2, ptr.numel,"");
	val=new array<type>(1,&ptr.numel, 0);
	for (c1=0;c1<ptr.numel;c1++) {		
		ind->data[c1*2+0]=ptr(c1)%a.size[0];
		ind->data[c1*2+1]=ptr(c1)/a.size[0];
		val->data[c1]		 =a(ptr(c1));
	};
	*ind*=3;
	memcpy(size,a.size,8);
};

template <class type> matrix<type> sparse<type>::full(void) {
	matrix<type> x(size[0],size[1],"");
	//bucket<unsigned int,type> b;
	//for (c1=0;c1<table->numel;c1++)
	//	while (table->data[c1]->token(b))
	//		x.data(b.key,c1)=b.data;
	return x;
};

template <class type> matrix<type> sparse<type>::multMatSparse(matrix<type> &a) {
	int c1,c2,cnt=0,i,j;
	matrix<float> x(a.size[0],size[1],"");	
	for (c1=0;c1<val->numel;c1++) {
		i=ind->data[cnt++];
		j=ind->data[cnt++];
		for (c2=0;c2<a.size[0];c2++)
			x.data[c2+j]+=a.data[c2+i]*val->data[c1];	
	}
	return x;
};
              


#endif