#ifndef jPhantom_H
#define jPhantom_H

#define GST_THROW  1
#define GST_TOGGLE 2
#define GST_DRAWLIST 4
#define KEY_GHOST 255

class jPhantom : public animated {
	public:
		jPhantom(int n); //servoLoop func,# of layouts
		void update(void);
		void control(bool human);	//merry christmas
		void draw(bool human);
		void setMode(int m);		
		void wand(int opt,void* ani);
		int ghost;						//identifier	
		gstVector pos,vel,sty,st0,ome,ang; //ang - joint angles / ome - stylus rot vel using st0
		gstVector force,glForce,glForceAdd; //force @ 1000Hz | glForce @ 70Hz (has to be set to 0 every frame)
		float heat[6];				//temperature
		gstTransformMatrix transform;	//temporary buffer for data read		
		array<unsigned int> *layout;	//[0=current layout| list handle of all layouts]
		array<float> *blend;  //blending
		matrix<float> **interpret;
	private:								//time[2] = delay for toggling modes
		int mode,cnt;					//bitwize: moving| maybe not private (to read!)
};
jPhantom::jPhantom(int n) : animated() {
	ghost=gstInitializePhantom("Default PHANToM");
	if (gstIsPhantomResetNeeded(ghost)) {
		cout << "Place jPhantom in reset position and press <ENTER>." << endl; cin.get();
		gstResetPhantomEncoders(ghost); }
	gstEnablePhantomForces(ghost);
	gstInitServoScheduler();
	blend       =new array<float>(1,new int(1000),0);
	for (int c1=0;c1<blend->numel;c1++) blend->data[c1]=.001*c1;
	layout      =new array<unsigned int>(1,&n,0);
	interpret   =new matrix<float>*[4];
	interpret[0]=new matrix<float>("data\\gst_styz.fil");
	interpret[1]=new matrix<float>(interpret[0]->numel,1,"");	
	//interpret[2]=new matrix<float>("data\\gst_cir.fil");
	//interpret[3]=new matrix<float>(interpret[2]->numel,1,"");	
	mode=0; cnt=0;
	time[2]=toc+1000;
	setMassAndG(1,0);
};
void jPhantom::update(void) {
	gstUpdatePhantom(ghost);
	gstGetStylusMatrix(ghost,transform);
	gstGetPhantomPosition(ghost,pos);
	gstGetPhantomVelocity(ghost,vel);
	gstGetStylusJointAngles(ghost,sty);
	force=gstVector(0,0.23,0); //anti gravity	
	if (mode&1) force-=pos*.07; 	
	if (mode&2) force-=vel*.002; //viscosity
	if (cnt++==10) {
		cnt=0;
		interpret[1]->record(sty.z());
		float sig=interpret[1]->filter(interpret[0]);
		int cyc=(sig> .28)-(sig<-.28);
		if (time[2]<toc&&cyc) {
			layout->cycle(cyc);
			time[2]=toc+400;
		}
	}
	force+=glForce;
	force*=blend->token(1);	
};
void jPhantom::setMode(int m) {
	if (mode!=m) {mode=m; blend->size[1]=0; time[2]=toc+300;}
};
void jPhantom::control(bool human) {//use human for different kind
	const double *temp;
	temp=transform.getOpenGLMatrix();
	for (int c1=0;c1<16;c1++) mat->data[c1]=(float)temp[c1];
	mat->data[14]-=128; for (c1=12;c1<15;c1++) mat->data[c1]*=.05; //never touch
	if (mode&1&&time[2]<toc) {
		((animated*)carrier)->oforce->data[0]+=pos.x();
		((animated*)carrier)->oforce->data[1]+=pos.y();
		((animated*)carrier)->oforce->data[2]+=pos.z();
		((animated*)carrier)->rforce->data[0]+=(sty.y()-M_PI)*10;  //use ->bound here!!!
		((animated*)carrier)->rforce->data[1]+=(sty.x()-M_PI)*15;
		((animated*)carrier)->rforce->data[2]+=(M_PI-sty.z())*4;
	};
};
void jPhantom::draw(bool human) {
	int bt=keys.data[KEY_GHOST];
	keys.data[KEY_GHOST]=gstGetStylusSwitchState(ghost);
	keys.data[KEY_GHOST+KEY_DOWN]=keys.data[KEY_GHOST]-bt;
	gstGetJointAngles(ghost,ang);
	gstGetPhantomVelocity(ghost,vel);
	ome=sty-st0; st0=sty;
	if (keys.data[KEY_ST+112]) {
		char glt[80];
		if (keys.data[KEY_ST+KEY_DOWN+45]) setMode(mode^1);
		gstGetPhantomTemperature(ghost,heat);
		sprintf(glt,"jPhantom@%6.1f",gstGetPhantomUpdateRate(ghost));
		linked_label.append(new glLabelText(0,frame_h-11,glt,fnt0));
		sprintf(glt,"vel: %7.2f,%7.2f,%7.2f",vel.x(),vel.y(),vel.z());
		linked_label.append(new glLabelText(0,frame_h-22,glt,fnt0));
		sprintf(glt,"pos: %7.2f,%7.2f,%7.2f",pos.x(),pos.y(),pos.z());		
		linked_label.append(new glLabelText(0,frame_h-33,glt,fnt0));
		sprintf(glt,"sty: %5.2f,%5.2f,%5.2f",sty.x(),sty.y(),sty.z());
		linked_label.append(new glLabelText(0,frame_h-44,glt,fnt0));
		sprintf(glt,"ang: %5.2f,%5.2f,%5.2f",ang.x(),ang.y(),ang.z());
		linked_label.append(new glLabelText(0,frame_h-55,glt,fnt0));		
		sprintf(glt,"heat: %4.1f,%4.1f,%4.1f",heat[0],heat[1],heat[2]);
		linked_label.append(new glLabelText(0,frame_h-66,glt,fnt0));		
	}	
	if (layout) {
		if (keys.data[KEY_GHOST]) glColor3f(.9,.1,.1); else glColor3f(.3,.1,.6);
		glutSolidSphere(.1,10,10);
		glCallList(layout->cycle(0));
	};	
	glForce=(glForce+glForceAdd)*.5;
	glForceAdd=gstVector(0,0,0);
};
void jPhantom::wand(int opt,void* ptr) {
	if (opt&GST_THROW)
		if (keys.data[KEY_GHOST]) {
			//setMode(mode|2);			
			animated *ani=(animated*)ptr;
			*ani->mat=*((animated*)carrier)->mat**mat*glTranslate(0,0,-4);
			memcpy(ani->tforce->data,zeros_float,16);
			memcpy(ani->oforce->data,zeros_float,16);		
			memcpy(ani->rforce->data,zeros_float,16);
			memcpy(ani->tvel->data,zeros_float,16);
			memcpy(ani->rvel->data,zeros_float,16);
			matrix<float> v(4,1,"");
			v.data[0]=vel.x(); v.data[1]=vel.y();	v.data[2]=vel.z();
			memcpy(ani->tvel->data,(*((animated*)carrier)->mat*v*.1).data,12);
			ani->rvel->data[0]= ome.y()*100;
			ani->rvel->data[1]= ome.x()*120;
			ani->rvel->data[2]=-ome.z()*30;
		} //else setMode(mode^2);
	//if (opt&GST_TOGGLE) {
	//	float signal=interpret[1]->filter(interpret[0]);
	//	interpret[1]->record(sty.z());
	//	if (time[2]<toc) {
	//		int cyc=(signal> .29)-(signal<-.28);
	//		layout->cycle(cyc);
	//		if (cyc) time[2]=toc+400;
	//	}
	//}
	if (opt&GST_DRAWLIST) {
		glPushMatrix();
			glMultMatrixf((*((animated*)carrier)->mat**mat*glTranslate(0,0,-4)).data);
			glCallList(*(unsigned int*)ptr);
		glPopMatrix();
	}
};
//------------------------------------------------------------------------
unsigned int jPhantom_listGstPtr1(unsigned int hnd) {//squished cone	
	int c1,n=17;
	float f=2*M_PI/(float)n,r1=.17,r2=.08,hi=1.04,ra=.9;
	glNewList(hnd,GL_COMPILE);
		glColor3f(.9,.8,.3);
		glBegin(GL_TRIANGLE_FAN);
			glNormal3f(0,0,-1);
			glVertex3f(0,0,0);
			for (c1=0;c1<=n;c1++) {
				glNormal3f(cos(c1*f),sin(c1*f),0);
				glVertex3f(r1*cos(c1*f),r2*sin(c1*f),1); } glEnd();
		glBegin(GL_TRIANGLE_FAN);
			glNormal3f(0,0,1);
			glVertex3f(0,0,hi+.02);
			for (c1=0;c1<=n;c1++) {
				glNormal3f(cos(c1*f),sin(c1*f),0);
				glVertex3f(r1*cos(c1*f)*ra,r2*sin(c1*f)*ra,hi);	} glEnd();
		glColor3f(.3,.1,0);
		glBegin(GL_QUAD_STRIP); for (c1=0;c1<=n;c1++) {
			glNormal3f(cos(c1*f),sin(c1*f),0);
			glVertex3f(r1*cos(c1*f)   ,r2*sin(c1*f)    ,1);
			glVertex3f(r1*cos(c1*f)*ra,r2*sin(c1*f)*ra,hi);}	glEnd();
	glEndList();
	return hnd; };
unsigned int jPhantom_listGstPtr2(unsigned int hnd) {//fine spray can
	int c1,n=14;
	float f=2*M_PI/(float)n,r1=.5,r2=.6;
	glNewList(hnd,GL_COMPILE);
		glPushMatrix();
			glScalef(.2,.4,.2);
			glColor3f(.9,.8,.3);
			glBegin(GL_QUAD_STRIP); for (c1=0;c1<=n;c1++)	{
				glNormal3f(cos(c1*f),0,sin(c1*f));
				glVertex3f(r1*cos(c1*f),  0,r1*sin(c1*f));	glVertex3f(   cos(c1*f),-.3,   sin(c1*f));} glEnd();
			glBegin(GL_QUAD_STRIP); for (c1=0;c1<=n;c1++)	{
				glNormal3f(cos(c1*f),0,sin(c1*f));
				glVertex3f(r2*cos(c1*f), -1,r2*sin(c1*f));	glVertex3f(   cos(c1*f),-.4,   sin(c1*f));}	glEnd();
			glColor3f(.3,.1,0);
			glBegin(GL_QUAD_STRIP); for (c1=0;c1<=n;c1++)	{
				glNormal3f(cos(c1*f),0,sin(c1*f));
				glVertex3f(   cos(c1*f),-.3,   sin(c1*f));	glVertex3f(   cos(c1*f),-.4,   sin(c1*f));}	glEnd();
			glNormal3f(0,-1,0);
			glBegin(GL_TRIANGLE_FAN); glVertex3f(0,-1,0); for (c1=0;c1<=n;c1++)	glVertex3f(r2*cos(c1*f), -1,r2*sin(c1*f)); glEnd();
		glPopMatrix();
	glEndList();
	return hnd; };
unsigned int jPhantom_listGstPtr3(unsigned int hnd) {//spray can
	int c1,n=14;
	float f=2*M_PI/(float)n;
	glNewList(hnd,GL_COMPILE);
		glPushMatrix();
			glScalef(.2,.4,.2);
			glColor3f(.9,.8,.3);
			glBegin(GL_QUAD_STRIP); for (c1=0;c1<=n;c1++)	{
				glNormal3f(cos(c1*f),0,sin(c1*f));
				glVertex3f(cos(c1*f), 0,sin(c1*f));	glVertex3f(cos(c1*f),-1,sin(c1*f));} glEnd();									
			glColor3f(.3,.1,0);
			glNormal3f(0, 1,0);
			glBegin(GL_TRIANGLE_FAN); glVertex3f(0, 0,0); for (c1=0;c1<=n;c1++)	glVertex3f(cos(c1*f), 0,sin(c1*f)); glEnd();
			glNormal3f(0,-1,0);
			glBegin(GL_TRIANGLE_FAN); glVertex3f(0,-1,0); for (c1=0;c1<=n;c1++)	glVertex3f(cos(c1*f),-1,sin(c1*f)); glEnd();
		glPopMatrix();
	glEndList();
	return hnd; };
void jPhantom_void0(void) {
	glScalef(.1,.1,1);
	glTranslatef(0,0,.5);	glutSolidCube(1);
	glTranslatef(0,0,.5);	glutSolidCone(1,.2,12,4);
};
unsigned int jPhantom_listGstPtr4(unsigned int hnd) {//spray can
	glNewList(hnd,GL_COMPILE);
		glPushMatrix();
			glPushMatrix(); glColor3f(1,0,0); glRotatef(90,0,1,0); 	jPhantom_void0(); glPopMatrix();
			glPushMatrix(); glColor3f(0,1,0); glRotatef(90,-1,0,0); jPhantom_void0(); glPopMatrix();
			glPushMatrix(); glColor3f(0,0,1); 											jPhantom_void0(); glPopMatrix();
			glColor3f(.8,.8,.8); glutSolidCube(.2);
		glPopMatrix();
	glEndList();
	return hnd; };

#endif