#ifndef ADAPTIVE_MESH_H
#define ADAPTIVE_MESH_H

#include "Mesh.h"
#include "ArrayMesh.h"
#include "AdaptiveArray.h"

/**
 * An adaptive array implementation of a mesh.  This class puts no
 * restrictions on the number of vertices or faces added
 *
 * @author Scott Schaefer
 */

class AdaptiveMesh : public Mesh
{
protected:
	// vertex storage
	AdaptiveArray<Vertex> *vertices;
public:
	// face storage
	AdaptiveArray<Face> *faces;

public:
	/**
	 * Constructor 
	 */
	AdaptiveMesh ( );

	/**
	 * Destructor
	 */
	~AdaptiveMesh ( void );

	/**
	 * @return the number of faces (triangles) stored in this mesh
	 */
	virtual int getNumFaces ( void );

	/**
	 * @return the number of vertices stored in this mesh
	 */
	int getNumVertices ( void );

	/**
	 * Adds a face of the mesh to the list of faces
	 *
	 * @param face an array of 3 ints that index into the vertex array
	 *             for this face
	 */
	virtual int addFace ( Face* face );

	/** 
	 * @return the face at the given index
	 */
	virtual Face* getFace ( int i );

	/**
	 * Adds a vertex to the list of vertices
	 *
	 * @param vertex a float array of size numComponents to copy in
	 */
	int addVertex ( Vertex *vertex );

	/**
	 * @return the ith vertex
	 */
	Vertex* getVertex ( int i );

	/**
	 * @return this mesh returned as an array mesh.  Array meshes are
	 *         typically faster for accessing than adaptive meshes
	 */
	virtual ArrayMesh *toArrayMesh ( void );
};

#endif