#ifndef ARRAY_MESH_H
#define ARRAY_MESH_H

#include "Mesh.h"

/**
 * An array implementation of the mesh.  Requires that you know the maximum
 * number of vertices and faces before the mesh is created.  It has faster
 * access time than AdaptiveMesh though.
 *
 * @author Scott Schaefer
 */

class ArrayMesh : public Mesh
{
protected:
	// storage for vertices
	Vertex *vertices;
	// storage for faces
	Face *faces;
	// total number of vertices
	int totalVerts;
	// total number of faces (triangles)
	int totalFaces;

	// current number of vertices
	int numVerts;
	// current number of faces
	int numFaces;
public:

	/**
	 * Constructor
	 *
	 * @param numFaces the maximum number of faces that will be stored in this mesh
	 * @param numVertices the maximum number of vertices that will be stored in this mesh
	 * @param numComponents the number of components per vertex to store
	 */
	ArrayMesh ( int numFaces, int numVertices );

	/**
	 * Destructor
	 */
	~ArrayMesh ( void );

	/**
	 * @return the number of faces (triangles) stored in this mesh
	 */
	int getNumFaces ( void );

	/**
	 * @return the number of vertices stored in this mesh
	 */
	int getNumVertices ( void );

	/**
	 * Adds a face of the mesh to the list of faces
	 *
	 * @param face an array of 3 ints that index into the vertex array
	 *             for this face
	 */
	virtual int addFace ( Face* face );

	/** 
	 * @return the face at the given index
	 */
	virtual Face* getFace ( int i );

	/**
	 * Adds a vertex to the list of vertices
	 *
	 * @param vertex a float array of size numComponents to copy in
	 */
	int addVertex ( Vertex *vertex );

	/**
	 * @return the ith vertex
	 */
	Vertex* getVertex ( int i );
};

#endif