#ifndef MESH_H_FILE
#define MESH_H_FILE

#include <string.h>
#include <stdio.h>
//#include "ArrayMesh.h"

/**
 * Abstract base class for a mesh.  This is essentially a list of
 * vertices and a list of faces that index into the vertex list
 *
 * @author Scott Schaefer
 */

class Face
{
public:
	int *data;
	int length;
	int select; //0 or 1
public:
	Face ( int length = 0 )
	{
		this->length = length;
		data = new int [ length ];
		memset ( data, 0, sizeof ( int ) * length );
		select=0;
	}

	Face ( const Face& f )
	{
		length = f.length;
		data = new int [ length ];

		memcpy ( data, f.data, sizeof ( int ) * length );
	}

	~Face ( void )
	{
		delete[] data;
	}

	int *getData ( void )
	{
		return data;
	}

	int getLength ( void ) const
	{
		return length;
	}

	int operator = ( Face &f )
	{
		copy ( &f );

		return 1;
	}

	void copy ( Face *f )
	{
//		if ( length != f->getLength ( ) )
		{
			delete[] data;

			length = f->getLength ( );
			data = new int [ length ];
			select=f->select;
		}

		memcpy ( data, f->getData ( ), length * sizeof ( int ) );		
	}
};

class Vertex
{
public:
	float *data;
	int length;
	int select;
public:
	Vertex ( int length = 8 )
	{
		this->length = length;
		data = new float [ length ];
		zero ( );
		select=0;
	}

	Vertex ( const Vertex &v )
	{
		length = v.length;
		data = new float [ length ];
		
		memcpy ( data, v.data, length * sizeof ( float ) );
	}

	~Vertex ( void )
	{
		delete[] data;
	}

	int operator = ( Vertex &v )
	{
		copy ( &v );

		return 1;
	}

	void copy ( Vertex *v )
	{
//		if ( v->getLength ( ) != length )
		{
			delete[] data;

			length = v->getLength ( );
			select=v->select;
			data = new float [ length ];
		}

		memcpy ( data, v->data, length * sizeof ( float ) );
	}

	void zero ( void )
	{
		int i;

		for ( i = 0; i < length; i++ )
		{
			data [ i ] = 0;
		}
	}

	float *getVert ( void )
	{
		return data;
	}

	int getLength ( void ) const
	{
		return length;
	}

	void print ( void ) const
	{
		int i;

		printf ( "{" );
		for ( i = 0; i < getLength ( ); i++ )
		{
			printf ( "%.2f ", data [ i ] );
		}
		printf ( "}\n" );
	}
};


class Mesh 
{
protected:
	// number of components per vertex

public:
	/**
	 * Constructor
	 */
	Mesh ( void )
	{
	}

	virtual ~Mesh ( void )
	{
	}

	Mesh* clone ( void );

	/**
	 * @return the number of faces (triangles) stored in this mesh
	 */
	virtual int getNumFaces ( void ) = 0;

	/**
	 * @return the number of vertices stored in this mesh
	 */
	virtual int getNumVertices ( void ) = 0;

	/**
	 * Adds a face of the mesh to the list of faces
	 *
	 * @param face an array of 3 ints that index into the vertex array
	 *             for this face
	 */
	virtual int addFace ( Face* face ) = 0;

	int addFace ( Face &face )
	{
		return addFace ( &face );
	}

	/** 
	 * @return the face at the given index
	 */
	virtual Face* getFace ( int i ) = 0;

	/**
	 * Adds a vertex to the list of vertices
	 *
	 * @param vertex a float array of size numComponents to copy in
	 */
	virtual int addVertex ( Vertex *vertex ) = 0;

	/**
	 * @return the ith vertex
	 */
	virtual Vertex* getVertex ( int i ) = 0;
};



#endif
