#include "TetrahedralSubdivision.h"
#include "TwoKeyHashTable.h"
#include "AdaptiveMesh.h"
#include "Math.h"

TetrahedralSubdivision *TetrahedralSubdivision::singletonInstance = NULL;

Mesh *TetrahedralSubdivision::subdivideMesh ( Mesh *mesh )
{
	Mesh *linear = linearSub ( mesh );
	Mesh *rvalue = average ( linear );

	delete linear;
	return rvalue;
}

int getVertex ( TwoKeyHashTable<int> &hash, int i1, int i2, Mesh *rvalue )
{
	int index;

	if ( i1 > i2 )
	{
		index = i1;
		i1 = i2;
		i2 = index;
	}

	if ( !hash.getElement ( i1, i2, index ) )
	{
		Vertex *v1 = rvalue->getVertex ( i1 );
		Vertex *v2 = rvalue->getVertex ( i2 );
		Vertex v ( v1->getLength ( ) );
		int i;

		for ( i = 0; i < v.getLength ( ); i++ ) 
		{
			v.getVert ( ) [ i ] = ( v1->getVert ( ) [ i ] + v2->getVert ( ) [ i ] ) / 2.0f;
		}

		index = rvalue->addVertex ( &v );
		hash.addElement ( i1, i2, index );
	}

	return index;
}

Mesh *TetrahedralSubdivision::linearSub ( Mesh *mesh )
{
	int i, j;
	AdaptiveMesh *rvalue = new AdaptiveMesh ( );
	TwoKeyHashTable<int> hash;
	int e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11, e12, middle, ab, ac, ad, bc, bd, cd;

	for ( i = 0; i < mesh->getNumVertices ( ); i++ )
	{
		rvalue->addVertex ( mesh->getVertex ( i ) );
	}

	for ( i = 0; i < mesh->getNumFaces ( ); i++ )
	{
		Face *f = mesh->getFace ( i );

		switch ( f->getLength ( ) )
		{
		case 1: // crease point
			rvalue->addFace ( f );
			break;
		case 2: // crease edge
			{
				middle = getVertex ( hash, f->getData ( ) [ 0 ], f->getData ( ) [ 1 ], rvalue );
				Face newF ( 2 );

				newF.getData ( ) [ 0 ] = f->getData ( ) [ 0 ];
				newF.getData ( ) [ 1 ] = middle;

				rvalue->addFace ( &newF );

				newF.getData ( ) [ 0 ] = middle;
				newF.getData ( ) [ 1 ] = f->getData ( ) [ 1 ];

				rvalue->addFace ( &newF );
			}
			break;
		case 3: // crease triangle
			{
				e1 = getVertex ( hash, f->getData ( ) [ 0 ], f->getData ( ) [ 1 ], rvalue );
				e2 = getVertex ( hash, f->getData ( ) [ 1 ], f->getData ( ) [ 2 ], rvalue );
				e3 = getVertex ( hash, f->getData ( ) [ 2 ], f->getData ( ) [ 0 ], rvalue );
				Face newF ( 3 );

				newF.getData ( ) [ 0 ] = f->getData ( ) [ 0 ];
				newF.getData ( ) [ 1 ] = e1;
				newF.getData ( ) [ 2 ] = e3;
				rvalue->addFace ( &newF );

				newF.getData ( ) [ 0 ] = f->getData ( ) [ 1 ];
				newF.getData ( ) [ 1 ] = e2;
				newF.getData ( ) [ 2 ] = e1;
				rvalue->addFace ( &newF );

				newF.getData ( ) [ 0 ] = f->getData ( ) [ 2 ];
				newF.getData ( ) [ 1 ] = e3;
				newF.getData ( ) [ 2 ] = e2;
				rvalue->addFace ( &newF );

				newF.getData ( ) [ 0 ] = e1;
				newF.getData ( ) [ 1 ] = e2;
				newF.getData ( ) [ 2 ] = e3;
				rvalue->addFace ( &newF );
			}
			break;
		case 4: // tetrahedra
			{
				ab = getVertex ( hash, f->getData ( ) [ 0 ], f->getData ( ) [ 1 ], rvalue );
				ac = getVertex ( hash, f->getData ( ) [ 0 ], f->getData ( ) [ 2 ], rvalue );
				ad = getVertex ( hash, f->getData ( ) [ 0 ], f->getData ( ) [ 3 ], rvalue );
				bc = getVertex ( hash, f->getData ( ) [ 1 ], f->getData ( ) [ 2 ], rvalue );
				bd = getVertex ( hash, f->getData ( ) [ 1 ], f->getData ( ) [ 3 ], rvalue );
				cd = getVertex ( hash, f->getData ( ) [ 2 ], f->getData ( ) [ 3 ], rvalue );
				Face newTet ( 4 );
				Face newOct ( 6 );

				newTet.getData ( ) [ 0 ] = f->getData ( ) [ 0 ];//good job!
				newTet.getData ( ) [ 1 ] = ab;
				newTet.getData ( ) [ 2 ] = ac;
				newTet.getData ( ) [ 3 ] = ad;
				rvalue->addFace ( &newTet );

				//newTet.getData ( ) [ 0 ] = f->getData ( ) [ 1 ];
				//newTet.getData ( ) [ 1 ] = bd;
				//newTet.getData ( ) [ 2 ] = bc;
				//newTet.getData ( ) [ 3 ] = ab;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = ab;
				newTet.getData ( ) [ 1 ] = f->getData ( ) [ 1 ];
				newTet.getData ( ) [ 2 ] = bc;                  
				newTet.getData ( ) [ 3 ] = bd;                  
				rvalue->addFace ( &newTet );				

				//newTet.getData ( ) [ 0 ] = f->getData ( ) [ 2 ];
				//newTet.getData ( ) [ 1 ] = bc;
				//newTet.getData ( ) [ 2 ] = cd;
				//newTet.getData ( ) [ 3 ] = ac;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = ac;
				newTet.getData ( ) [ 1 ] = bc;
				newTet.getData ( ) [ 2 ] = f->getData ( ) [ 2 ];
				newTet.getData ( ) [ 3 ] = cd;
				rvalue->addFace ( &newTet );				

				//newTet.getData ( ) [ 0 ] = f->getData ( ) [ 3 ];
				//newTet.getData ( ) [ 1 ] = cd;
				//newTet.getData ( ) [ 2 ] = bd;
				//newTet.getData ( ) [ 3 ] = ad;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = ad;
				newTet.getData ( ) [ 1 ] = bd;
				newTet.getData ( ) [ 2 ] = cd;
				newTet.getData ( ) [ 3 ] = f->getData ( ) [ 3 ];
				rvalue->addFace ( &newTet );
				
				//newOct.getData ( ) [ 0 ] = ab;
				//newOct.getData ( ) [ 1 ] = ac;
				//newOct.getData ( ) [ 2 ] = ad;
				//newOct.getData ( ) [ 3 ] = cd;
				//newOct.getData ( ) [ 4 ] = bd;
				//newOct.getData ( ) [ 5 ] = bc;
				//rvalue->addFace ( &newOct );
				newOct.getData ( ) [ 0 ] = bc;
				newOct.getData ( ) [ 1 ] = cd;
				newOct.getData ( ) [ 2 ] = bd;
				newOct.getData ( ) [ 3 ] = ad;
				newOct.getData ( ) [ 4 ] = ab;
				newOct.getData ( ) [ 5 ] = ac;
				rvalue->addFace ( &newOct );
			}
			break;
		case 6: // octahedra {a,b,c,d,e,f}  {a,d}, {b,e}, {c,f} are opposite vertices
			{
				e1 = getVertex ( hash, f->getData ( ) [ 0 ], f->getData ( ) [ 1 ], rvalue );
				e2 = getVertex ( hash, f->getData ( ) [ 0 ], f->getData ( ) [ 2 ], rvalue );
				e3 = getVertex ( hash, f->getData ( ) [ 0 ], f->getData ( ) [ 4 ], rvalue );
				e4 = getVertex ( hash, f->getData ( ) [ 0 ], f->getData ( ) [ 5 ], rvalue );
				e5 = getVertex ( hash, f->getData ( ) [ 1 ], f->getData ( ) [ 2 ], rvalue );
				e6 = getVertex ( hash, f->getData ( ) [ 2 ], f->getData ( ) [ 4 ], rvalue );
				e7 = getVertex ( hash, f->getData ( ) [ 4 ], f->getData ( ) [ 5 ], rvalue );
				e8 = getVertex ( hash, f->getData ( ) [ 5 ], f->getData ( ) [ 1 ], rvalue );
				e9 = getVertex ( hash, f->getData ( ) [ 3 ], f->getData ( ) [ 1 ], rvalue );
				e10 = getVertex ( hash, f->getData ( ) [ 3 ], f->getData ( ) [ 2 ], rvalue );
				e11 = getVertex ( hash, f->getData ( ) [ 3 ], f->getData ( ) [ 4 ], rvalue );
				e12 = getVertex ( hash, f->getData ( ) [ 3 ], f->getData ( ) [ 5 ], rvalue );
				Face newTet ( 4 );
				Face newOct ( 6 );

				Vertex *v1, *v2, *v3, *v4, *v5, *v6;
				v1 = mesh->getVertex ( f->getData ( ) [ 0 ] );
				v2 = mesh->getVertex ( f->getData ( ) [ 1 ] );
				v3 = mesh->getVertex ( f->getData ( ) [ 2 ] );
				v4 = mesh->getVertex ( f->getData ( ) [ 3 ] );
				v5 = mesh->getVertex ( f->getData ( ) [ 4 ] );
				v6 = mesh->getVertex ( f->getData ( ) [ 5 ] );
				Vertex v ( v1->getLength ( ) );

				for ( j = 0; j < v.getLength ( ); j++ )
				{
					v.getVert ( ) [ j ] = ( v1->getVert ( ) [ j ] + v2->getVert ( ) [ j ] + v3->getVert ( ) [ j ] +
										v4->getVert ( ) [ j ] + v5->getVert ( ) [ j ] + v6->getVert ( ) [ j ] ) / 6.0f;
				}
				middle = rvalue->addVertex ( &v );
				
				newOct.getData ( ) [ 0 ] = f->getData ( ) [ 0 ]; //good job!
				newOct.getData ( ) [ 1 ] = e1;
				newOct.getData ( ) [ 2 ] = e2;
				newOct.getData ( ) [ 3 ] = middle;
				newOct.getData ( ) [ 4 ] = e3;
				newOct.getData ( ) [ 5 ] = e4;
				rvalue->addFace ( &newOct );//0
				
				//newOct.getData ( ) [ 0 ] = f->getData ( ) [ 1 ];
				//newOct.getData ( ) [ 1 ] = e5;
				//newOct.getData ( ) [ 2 ] = e1;
				//newOct.getData ( ) [ 3 ] = middle;
				//newOct.getData ( ) [ 4 ] = e8;
				//newOct.getData ( ) [ 5 ] = e9;
				//rvalue->addFace ( &newOct );				
				newOct.getData ( ) [ 0 ] = e1;
				newOct.getData ( ) [ 1 ] = f->getData ( ) [ 1 ];
				newOct.getData ( ) [ 2 ] = e5;
				newOct.getData ( ) [ 3 ] = e9;
				newOct.getData ( ) [ 4 ] = middle;
				newOct.getData ( ) [ 5 ] = e8;
				rvalue->addFace ( &newOct );//1
				
				//newOct.getData ( ) [ 0 ] = f->getData ( ) [ 2 ];
				//newOct.getData ( ) [ 1 ] = e6;
				//newOct.getData ( ) [ 2 ] = e2;
				//newOct.getData ( ) [ 3 ] = middle;
				//newOct.getData ( ) [ 4 ] = e5;
				//newOct.getData ( ) [ 5 ] = e10;
				//rvalue->addFace ( &newOct );
				newOct.getData ( ) [ 0 ] = e2;
				newOct.getData ( ) [ 1 ] = e5;
				newOct.getData ( ) [ 2 ] = f->getData ( ) [ 2 ];
				newOct.getData ( ) [ 3 ] = e10;
				newOct.getData ( ) [ 4 ] = e6;
				newOct.getData ( ) [ 5 ] = middle;
				rvalue->addFace ( &newOct );//2

				//newOct.getData ( ) [ 0 ] = f->getData ( ) [ 3 ];
				//newOct.getData ( ) [ 1 ] = e9;
				//newOct.getData ( ) [ 2 ] = e12;
				//newOct.getData ( ) [ 3 ] = middle;
				//newOct.getData ( ) [ 4 ] = e11;
				//newOct.getData ( ) [ 5 ] = e10;
				//rvalue->addFace ( &newOct );
				newOct.getData ( ) [ 0 ] = middle;
				newOct.getData ( ) [ 1 ] = e9;
				newOct.getData ( ) [ 2 ] = e10;
				newOct.getData ( ) [ 3 ] = f->getData ( ) [ 3 ];
				newOct.getData ( ) [ 4 ] = e11;
				newOct.getData ( ) [ 5 ] = e12;
				rvalue->addFace ( &newOct );//3			
				
				//newOct.getData ( ) [ 0 ] = f->getData ( ) [ 4 ];
				//newOct.getData ( ) [ 1 ] = e11;
				//newOct.getData ( ) [ 2 ] = e7;
				//newOct.getData ( ) [ 3 ] = middle;
				//newOct.getData ( ) [ 4 ] = e3;
				//newOct.getData ( ) [ 5 ] = e6;
				//rvalue->addFace ( &newOct );
				newOct.getData ( ) [ 0 ] = e3;
				newOct.getData ( ) [ 1 ] = middle;
				newOct.getData ( ) [ 2 ] = e6;
				newOct.getData ( ) [ 3 ] = e11;
				newOct.getData ( ) [ 4 ] = f->getData ( ) [ 4 ];
				newOct.getData ( ) [ 5 ] = e7;
				rvalue->addFace ( &newOct );//4
				
				//newOct.getData ( ) [ 0 ] = f->getData ( ) [ 5 ];
				//newOct.getData ( ) [ 1 ] = e8;
				//newOct.getData ( ) [ 2 ] = e4;
				//newOct.getData ( ) [ 3 ] = middle;
				//newOct.getData ( ) [ 4 ] = e7;
				//newOct.getData ( ) [ 5 ] = e12;
				//rvalue->addFace ( &newOct );				
				newOct.getData ( ) [ 0 ] = e4;
				newOct.getData ( ) [ 1 ] = e8;
				newOct.getData ( ) [ 2 ] = middle;
				newOct.getData ( ) [ 3 ] = e12;
				newOct.getData ( ) [ 4 ] = e7;
				newOct.getData ( ) [ 5 ] = f->getData ( ) [ 5 ];
				rvalue->addFace ( &newOct );//5			

				//newTet.getData ( ) [ 0 ] = middle;
				//newTet.getData ( ) [ 1 ] = e4;
				//newTet.getData ( ) [ 2 ] = e8;
				//newTet.getData ( ) [ 3 ] = e1;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = e4;
				newTet.getData ( ) [ 1 ] = e8;
				newTet.getData ( ) [ 2 ] = middle;
				newTet.getData ( ) [ 3 ] = e1;
				rvalue->addFace ( &newTet );//0

				//newTet.getData ( ) [ 0 ] = middle;
				//newTet.getData ( ) [ 1 ] = e1;
				//newTet.getData ( ) [ 2 ] = e5;
				//newTet.getData ( ) [ 3 ] = e2;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = e1;
				newTet.getData ( ) [ 1 ] = e2;
				newTet.getData ( ) [ 2 ] = e5;
				newTet.getData ( ) [ 3 ] = middle;
				rvalue->addFace ( &newTet );//1


				//newTet.getData ( ) [ 0 ] = middle;
				//newTet.getData ( ) [ 1 ] = e2;
				//newTet.getData ( ) [ 2 ] = e6;
				//newTet.getData ( ) [ 3 ] = e3;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = e3;
				newTet.getData ( ) [ 1 ] = middle;
				newTet.getData ( ) [ 2 ] = e6;
				newTet.getData ( ) [ 3 ] = e2;
				rvalue->addFace ( &newTet );//2
				

				//newTet.getData ( ) [ 0 ] = middle;
				//newTet.getData ( ) [ 1 ] = e3;
				//newTet.getData ( ) [ 2 ] = e7;
				//newTet.getData ( ) [ 3 ] = e4;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = e4;
				newTet.getData ( ) [ 1 ] = e3;
				newTet.getData ( ) [ 2 ] = middle;
				newTet.getData ( ) [ 3 ] = e7;
				rvalue->addFace ( &newTet );//3
				
				//newTet.getData ( ) [ 0 ] = middle;
				//newTet.getData ( ) [ 1 ] = e9;
				//newTet.getData ( ) [ 2 ] = e5;
				//newTet.getData ( ) [ 3 ] = e10;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = middle;
				newTet.getData ( ) [ 1 ] = e9;
				newTet.getData ( ) [ 2 ] = e10;
				newTet.getData ( ) [ 3 ] = e5;
				rvalue->addFace ( &newTet );//4 up


				//newTet.getData ( ) [ 0 ] = middle;
				//newTet.getData ( ) [ 1 ] = e10;
				//newTet.getData ( ) [ 2 ] = e6;
				//newTet.getData ( ) [ 3 ] = e11;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = middle;
				newTet.getData ( ) [ 1 ] = e6;
				newTet.getData ( ) [ 2 ] = e10;
				newTet.getData ( ) [ 3 ] = e11;
				rvalue->addFace ( &newTet );//5 down


				//newTet.getData ( ) [ 0 ] = middle;
				//newTet.getData ( ) [ 1 ] = e11;
				//newTet.getData ( ) [ 2 ] = e7;
				//newTet.getData ( ) [ 3 ] = e12;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = e7;
				newTet.getData ( ) [ 1 ] = e12;
				newTet.getData ( ) [ 2 ] = e11;
				newTet.getData ( ) [ 3 ] = middle;
				rvalue->addFace ( &newTet );//6 up

				//newTet.getData ( ) [ 0 ] = middle;
				//newTet.getData ( ) [ 1 ] = e12;
				//newTet.getData ( ) [ 2 ] = e8;
				//newTet.getData ( ) [ 3 ] = e9;
				//rvalue->addFace ( &newTet );
				newTet.getData ( ) [ 0 ] = e8;
				newTet.getData ( ) [ 1 ] = middle;
				newTet.getData ( ) [ 2 ] = e9;
				newTet.getData ( ) [ 3 ] = e12;
				rvalue->addFace ( &newTet );//7 down
			}
			break;
		}
	}

	return rvalue;
}

Mesh *TetrahedralSubdivision::average ( Mesh *mesh )
{
	ArrayMesh *rvalue = new ArrayMesh ( mesh->getNumFaces ( ), mesh->getNumVertices ( ) );
	int i, j, k;
	Vertex v ( mesh->getVertex ( 0 )->getLength ( ) );
	float *weight = new float [ mesh->getNumVertices ( ) ];
	int *dim = new int [ mesh->getNumVertices ( ) ];

	for ( i = 0; i < mesh->getNumVertices ( ); i++ )
	{
		rvalue->addVertex ( &v );
		dim [ i ] = 6;
		weight [ i ] = 0;
	}

	for ( i = 0; i < mesh->getNumFaces ( ); i++ )
	{
		Face *f = mesh->getFace ( i );
		rvalue->addFace ( f );

		for ( j = 0; j < f->getLength ( ); j++ )
		{
			if ( dim [ f->getData ( ) [ j ] ] > f->getLength ( ) )
			{
				dim [ f->getData ( ) [ j ] ] = f->getLength ( );
			}
		}
	}

	for ( i = 0; i < mesh->getNumFaces ( ); i++ )
	{
		Face *f = mesh->getFace ( i );

		switch ( f->getLength ( ) )
		{
		case 1:
			*rvalue->getVertex ( f->getData ( ) [ 0 ] ) = *mesh->getVertex ( f->getData ( ) [ 0 ] );
			weight [ f->getData ( ) [ 0 ] ] = 1;
			break;
		case 2:
			for ( j = 0; j < v.getLength ( ); j++ )
			{
				v.getVert ( ) [ j ] = ( mesh->getVertex ( f->getData ( ) [ 0 ] )->getVert ( ) [ j ] + 
										mesh->getVertex ( f->getData ( ) [ 1 ] )->getVert ( ) [ j ] ) / 2.0f;
			}

			if ( dim [ f->getData ( ) [ 0 ] ] == 2 )
			{
				for ( j = 0; j < v.getLength ( ); j++ )
				{
					rvalue->getVertex ( f->getData ( ) [ 0 ] )->getVert ( ) [ j ] += v.getVert ( ) [ j ];
				}
				weight [ f->getData ( ) [ 0 ] ]++;
			}

			if ( dim [ f->getData ( ) [ 1 ] ] == 2 )
			{
				for ( j = 0; j < v.getLength ( ); j++ )
				{
					rvalue->getVertex ( f->getData ( ) [ 1 ] )->getVert ( ) [ j ] += v.getVert ( ) [ j ];
				}
				weight [ f->getData ( ) [ 1 ] ]++;
			}
			break;
		case 3:
			for ( j = 0; j < 3; j++ )
			{
				if ( dim [ f->getData ( ) [ j ] ] == 3 )
				{
					for ( k = 0; k < v.getLength ( ); k++ )
					{
						rvalue->getVertex ( f->getData ( ) [ j ] )->getVert ( ) [ k ] += 
							( 0.25f * mesh->getVertex ( f->getData ( ) [ j ] )->getVert ( ) [ k ] + 
							0.375f * mesh->getVertex ( f->getData ( ) [ ( j + 1 ) % 3 ] )->getVert ( ) [ k ] + 
							0.375f * mesh->getVertex ( f->getData ( ) [ ( j + 2 ) % 3 ] )->getVert ( ) [ k ] );
					}
					weight [ f->getData ( ) [ j ] ]++;
				}
			}
			break;
		case 4:
			for ( j = 0; j < 4; j++ )
			{
				if ( dim [ f->getData ( ) [ j ] ] >= 4 )
				{
					for ( k = 0; k < v.getLength ( ); k++ )
					{
						rvalue->getVertex ( f->getData ( ) [ j ] )->getVert ( ) [ k ] += 
							( -0.0625f * mesh->getVertex ( f->getData ( ) [ j ] )->getVert ( ) [ k ] + 
							0.354167f * mesh->getVertex ( f->getData ( ) [ ( j + 1 ) % 3 ] )->getVert ( ) [ k ] + 
							0.354167f * mesh->getVertex ( f->getData ( ) [ ( j + 2 ) % 3 ] )->getVert ( ) [ k ] + 
							0.354167f * mesh->getVertex ( f->getData ( ) [ ( j + 3 ) % 3 ] )->getVert ( ) [ k ] );
					}
					weight [ f->getData ( ) [ j ] ]++;
				}
			}
			break;
		case 6:
			for ( j = 0; j < 6; j++ )
			{
				if ( dim [ f->getData ( ) [ j ] ] >= 4 )
				{
					for ( k = 0; k < v.getLength ( ); k++ )
					{
						rvalue->getVertex ( f->getData ( ) [ j ] )->getVert ( ) [ k ] += 
							( 0.375f * mesh->getVertex ( f->getData ( ) [ j ] )->getVert ( ) [ k ] + 
							0.08333333f * mesh->getVertex ( f->getData ( ) [ ( j + 1 ) % 3 ] )->getVert ( ) [ k ] + 
							0.08333333f * mesh->getVertex ( f->getData ( ) [ ( j + 2 ) % 3 ] )->getVert ( ) [ k ] + 
							0.291667f * mesh->getVertex ( f->getData ( ) [ ( j + 3 ) % 3 ] )->getVert ( ) [ k ] + 
							0.08333333f * mesh->getVertex ( f->getData ( ) [ ( j + 4 ) % 3 ] )->getVert ( ) [ k ] + 
							0.08333333f * mesh->getVertex ( f->getData ( ) [ ( j + 5 ) % 3 ] )->getVert ( ) [ k ] );
					}
					weight [ f->getData ( ) [ j ] ]++;
				}
			}
			break;
		}
	}

	for ( i = 0; i < mesh->getNumVertices ( ); i++ )
	{
		Vertex *vert = rvalue->getVertex ( i );

		for ( j = 0; j < vert->getLength ( ); j++ )
		{
			vert->getVert ( ) [ j ] /= weight [ i ];
		}

		if ( dim [ i ] == 3 && weight [ i ] != 6 )
		{
			for ( j = 0; j < vert->getLength ( ); j++ )
			{
				vert->getVert ( ) [ j ] = mesh->getVertex ( i )->getVert ( ) [ j ] + 
					( 5.0f / 3.0f - ( 8.0f / 3.0f ) * ( 3.0f / 8.0f + 0.25f * (float)cos ( 2 * 3.141592654 / weight [ i ] ) ) * ( 3.0f / 8.0f + 0.25f * (float)cos ( 2 * 3.141592654 / weight [ i ] ) ) ) * 
					( vert->getVert ( ) [ j ] - mesh->getVertex ( i )->getVert ( ) [ j ] );
			}
		}
	}

	delete[] weight;
	delete[] dim;

	return rvalue;
}
