#ifndef TWO_KEY_HASH_TABLE_H
#define TWO_KEY_HASH_TABLE_H

#include "LList.h"

template <class Type>
class TwoKeyHashTable
{
public:
	struct HashData
	{
		Type data;
		int key1;
		int key2;

		HashData ( void )
		{
		}
	};

protected:
	LList<HashData> **buckets;
	int numBuckets;

public:
	TwoKeyHashTable ( void )
	{ 
		int i;

		numBuckets = 16381;

		buckets = new LList<HashData>* [ numBuckets ];

		for ( i = 0; i < numBuckets; i++ )
		{
			buckets [ i ] = NULL;
		}
	}

	~TwoKeyHashTable ( void )
	{
		LList<HashData> *ptr1, *ptr2;
		int i;

		for ( i = 0; i < numBuckets; i++ )
		{
			ptr1 = buckets [ i ];

			while ( ptr1 != NULL )
			{
				ptr2 = ptr1->getNext ( );

				delete ptr1;

				ptr1 = ptr2;
			}
		}

		delete[] buckets;
	}

	void clear ( void )
	{
		LList<HashData> *ptr1, *ptr2;
		int i;

		for ( i = 0; i < numBuckets; i++ )
		{
			ptr1 = buckets [ i ];

			while ( ptr1 != NULL )
			{
				ptr2 = ptr1->getNext ( );

				delete ptr1;

				ptr1 = ptr2;
			}
		}

		for ( i = 0; i < numBuckets; i++ )
		{
			buckets [ i ] = NULL;
		}
	}

	int hash ( int key1, int key2 )
	{
		return ( key1 + key2 ) % numBuckets;
	}

	bool contains ( int key1, int key2 )
	{
		Type crap;

		return getElement ( key1, key2, crap );
	}

	bool getElement ( int key1, int key2, Type &rvalue )
	{
		int index = hash ( key1, key2 );

		if ( buckets [ index ] == NULL )
		{
			// we haven't added any data to this bucket yet
			return false;
		}
		else
		{
			LList<HashData> *ptr = buckets [ index ];

			while ( ptr != NULL )
			{
				if ( ptr->getData ( ).key1 == key1 && ptr->getData ( ).key2 == key2 )
				{
					rvalue = ptr->getData ( ).data;
					return true;
				}

				ptr = ptr->getNext ( );
			}

			// bad mojo, didn't find it, add a node onto the front of the list
			return false;
		}
	}

	void addElement ( int key1, int key2, Type& value )
	{
		HashData myData;

		int index = hash ( key1, key2 );

		myData.key1 = key1;
		myData.key2 = key2;
		myData.data = value;

		buckets [ index ] = new LList<HashData> ( myData, buckets [ index ] );
	}
};

#endif