#ifndef MATRIX_H
#define MATRIX_H

#include "array.h"
#define M_EPS 1E-20
#define M_INF 1E308

float fsign(float a) {
	if 			(a>0) return 1;
	else if	(a<0) return -1;
	else return 0;
};

template<class type> const type& MIN(const type& a,const type& b) {
	return (a<b)? a: b;};
template<class type> const type& MAX(const type& a,const type& b) {
	return (a>b)? a: b;};
template<class type> type SIGN	(const type& a) {return (a<0) ?   -1 :    1; };
template<class type> int  ROUND	(const type& a) {return (a>0) ? a+.5 : a-.5; };
template<class type> type ABS		(const type& a) {return (a>0) ?    a :   -a; };

	
template <class type> class matrix : public array<type> {
	public:
		matrix<type>(void) : array<type>() {};
		matrix<type>(int m,int n,char *c);			//various constructors
		matrix<type>(char *fn);									//read from file [from array.h]
		matrix<type> diag(void);
		matrix<type> T(void);										//LATER TO BE replaced by bool ???
		matrix<type> solve(matrix<type> b);			//solve LGS
		//matrix<type> tridiag(matrix<type> b);		//solve tridiag LGS stored as nx3: (left|diag|right) matrix gets trashed !!!
		matrix<type> inv2x2(void);							//invert 2x2 LGS
		matrix<type> glInverse(void);						//invert 4x4 openGL matrix
		type glDet(void);
		type filter(matrix<type> *f);						//very spec but easy func (see explanation below)
		type normalize(void);										//normalize (vector) and return previous norm
		type norm(int n);												//n-norm of vector
		type mean(void);												//arithmetic mean
		matrix<type> colmean(void);
		matrix<type> operator+ (matrix<type>& a); //matrix add
		matrix<type> operator- (matrix<type>& a); //matrix-matrix
		matrix<type> operator* (matrix<type>& a); //matrix mult		
		matrix<type> operator+ (type a); 					//matrix+a
		matrix<type> operator- (type a); 					//matrix-a
		matrix<type> operator* (type a); 					//matrix*a
		matrix<type> operator% (matrix<type>& a); //pointwise product!!!
		matrix<type> operator& (matrix<type>& a); //cross product
		type         operator| (matrix<type>& a); //scalar product
		matrix<type> operator> (type a); 					//matrix gt a
		void operator+=(matrix<type>& a);				//matrix+a
		void operator-=(matrix<type>& a);				//matrix-a
		void operator*=(matrix<type>& a);				//matrix mult
		void operator%=(matrix<type>& a);				//pointwise prod
		void operator&=(matrix<type>& a);				//cross prod
		void operator+=(type a);								//matrix+a
		void operator-=(type a);								//matrix-a
		void operator*=(type a);								//matrix*a
		void operator/=(type a);								//a/matrix		
		bool operator==(array<type>& a);
		//void* operator[](int a) {return &data[a];};					//just to try!!		
		//matrix<type> operator[](const int& a);
};
template <class type> matrix<type>::matrix(int m,int n,char *c) {
	int c1;
	ndims=2; size=new int[3];
	size[0]=m; size[1]=n; size[2]=0; //tokenizer
	numel=m*n;
	data=new type[numel];
	if (!c[0]) 										for (c1=0;c1<numel;c1++) data[c1]=0;
	if (!stricmp(c,"fill")) 			for (c1=0;c1<numel;c1++) data[c1]=c1;
	else if (!stricmp(c,"ones")) 	for (c1=0;c1<numel;c1++) data[c1]=1;
	else if (!stricmp(c,"eye")) {
		for (c1=0;c1<numel;c1++) data[c1]=0;
		for (c1=0;c1<numel;c1+=n+1) data[c1]=1;
	}
	else if (!stricmp(c,"nchoosek"))
		if (n==1) {
			data[0]=1;
			for (c1=1;c1<m;c1++) data[c1]=data[c1-1]*(m-c1)/c1;
		}
};
template <class type> matrix<type>::matrix(char *fn) : array<type>(fn) {};
//:--------------------------------------------------------------
template <class type> matrix<type> matrix<type>::diag(void) {	
	matrix<float> x(numel,numel,"");
	for (int c1=0;c1<numel*numel;c1+=numel+1) x.data[c1]=token();
	return x;
}	
template <class type> matrix<type> matrix<type>::T(void) {
	int c1,c2,cnt=0;
	matrix<float> x(size[1],size[0],"");
	for (c1=0;c1<size[1];c1++)
		for (c2=0;c2<numel;c2+=size[1]) x.data[c1+c2]=data[cnt++];
	return x;
}	
template <class type> matrix<type> matrix<type>::solve(matrix<type> b) {
	int c1,c2,c3,c1s=0,k;
	type tfac,tmax,tabs;	
	matrix<float> x(b.size[0],b.size[1],"");
	matrix<int> p(size[0]+1,1,"fill");
	type *datacpy=new type[numel];
	memcpy(datacpy,data,sizeof(type)*numel);
	for (c1=0;c1<size[0];c1++) {
		k=c1; tmax=0;
		for (c2=c1;c2<size[0];c2++)
			if (tmax<(tabs=abs(datacpy[p.data[c2]+c1s]))) {tmax=tabs; k=c2;}
		p.data[size[0]]=p.data[k];
					p.data[k]=p.data[c1];
				 p.data[c1]=p.data[size[0]];
		for (c2=c1+1;c2<size[0];c2++) {
			tabs=datacpy[p.data[c2]+c1s]/datacpy[p.data[c1]+c1s];
			for (c3=(c1+1)*size[0];c3<numel;c3+=size[0]) datacpy[p.data[c2]+c3]-=datacpy[p.data[c1]+c3]*tabs;
			for (c3=0;c3<b.numel;c3+=size[0]) b.data[p.data[c2]+c3]-=b.data[p.data[c1]+c3]*tabs;
		}
		c1s+=size[0];
	}
	for (c1=size[0]-1;c1>=0;c1--) {
		tfac=1/datacpy[p.data[c1]+c1*size[0]];
		for (c2=0;c2<b.size[1];c2++) {			
			tabs=b.data[p.data[c1]+c2*size[0]];			
			for (c3=c1+1;c3<size[0];c3++) 
				tabs-=datacpy[p.data[c1]+c3*size[0]]*x.data[c3+c2*size[0]];
			x.data[c1+c2*size[0]]=tabs*tfac;
		}
	}	
	delete[] datacpy;
	return x;
};
template <class type> matrix<type> matrix<type>::inv2x2(void) {
	matrix<float> x(2,2,"");
	float det=data[0]*data[3]-data[1]*data[2];
	//printf("det: %f \n",det);
	if (fabs(det)>M_EPS) {
		det=1/det;
		x.data[0]= data[3]*det;
		x.data[1]=-data[1]*det;
		x.data[2]=-data[2]*det;
		x.data[3]= data[0]*det;
	}	//else matrix_safe=false;
	return x;
};
template <class type> matrix<type> matrix<type>::glInverse(void) {
	int c1,c2; float sum;
	matrix<float> inv(4,4,"");
	for (c1=0;c1<3;c1++)
		for (c2=0;c2<3;c2++) inv.data[c1*4+c2]=data[c2*4+c1];		//R'
	for (c1=0;c1<3;c1++) {sum=0;
		for (c2=0;c2<3;c2++) sum-=inv.data[c1+c2*4]*data[12+c2];//-R'*p
		inv.data[12+c1]=sum; };
	inv.data[15]=1;
	return inv;
};
//---------------------------
template <class type> type matrix<type>::filter(matrix<type> *f) {//this.records a arbitrary signal over the time (this is a "ring buffer") // f is a filter of the same numels as this
	int c1,cnt=0;	type sum=0;	
	for (c1=size[ndims];c1<numel;c1++) sum+=data[c1]*f->data[cnt++];
	for (c1=0;    c1<size[ndims];c1++) sum+=data[c1]*f->data[cnt++];
	return sum;
};
template <class type> type matrix<type>::norm(int n) {
	int c1;
	type f=0;
	switch (n) {
	case 0: //infinity norm
		for (c1=0;c1<numel;c1++) f=MAX<type>(ABS(data[c1]),f);
		return f;
	case 1:
		for (c1=0;c1<numel;c1++) f+=ABS(data[c1]);
		return f;
	case 2:
		for (c1=0;c1<numel;c1++) f+=data[c1]*data[c1];
		return sqrt(f);	
	}	
	return 0;
};

template <class type> type matrix<type>::normalize(void) {
	type f=0,fi;
	for (int c1=0;c1<numel;c1++) f+=data[c1]*data[c1];	
	if (f>M_EPS) {
		f=sqrt(f);
		fi=1/f;
		for (c1=0;c1<numel;c1++) data[c1]*=fi;
		return f;
	}	else return 0;
};
template <class type> type matrix<type>::mean(void) {
	type tmp=0;
	for (int c1=0;c1<numel;c1++) tmp+=data[c1];
	return tmp/numel;
};
template <class type> matrix<type> matrix<type>::colmean(void) {
	int c1,c2;
	matrix<type> a(size[0],1,"");
	for (c1=0;c1<size[0];c1++)
	for (c2=c1;c2<numel;c2+=size[0]) a.data[c1]+=data[c2];
	a*=1/(type)size[1];	
	return a;
};
//cunstructive operators-----------------------------------------------------------------------
template <class type> matrix<type> matrix<type>::operator+ (matrix<type>& a) {
	matrix<type> b=*this;
	for (int c1=0;c1<MIN(numel,a.numel);c1++) b.data[c1]+=a.data[c1];
	return b; };
template <class type> matrix<type> matrix<type>::operator- (matrix<type>& a) {
	matrix<type> b=*this;
	for (int c1=0;c1<MIN(numel,a.numel);c1++) b.data[c1]-=a.data[c1];
	return b; };
template <class type> matrix<type> matrix<type>::operator* (matrix<type>& a) {
	int c1,c2,c3;
	matrix<type> b(size[0],a.size[1],"");
	for (c1=0;c1<size[0];c1++)
		for (c2=0;c2<a.size[1];c2++)
			for (c3=0;c3<size[1];c3++)
				b.data[c1+c2*size[0]]+=data[c1+c3*size[0]]*a.data[c2*a.size[0]+c3];	
	return b; };
template <class type> matrix<type> matrix<type>::operator+ (type a) {
	matrix<type> b=*this;
	for (int c1=0;c1<numel;c1++) b.data[c1]+=a;
	return b; };
template <class type> matrix<type> matrix<type>::operator- (type a) {
	matrix<type> b=*this;
	for (int c1=0;c1<numel;c1++) b.data[c1]-=a;
	return b; };
template <class type> matrix<type> matrix<type>::operator* (type a) {
	matrix<type> b=*this;
	for (int c1=0;c1<numel;c1++) b.data[c1]*=a;
	return b; };
template <class type> matrix<type> matrix<type>::operator%(matrix<type>& a) {
	matrix<type> b=*this;
	for (int c1=0;c1<MIN(numel,a.numel);c1++) b.data[c1]*=a.data[c1];
	return b; };	
template <class type> matrix<type> matrix<type>::operator& (matrix<type>& a) {
	matrix<type> b(size[0],size[1],"");
	b.data[0]=a.data[2]*data[1]-a.data[1]*data[2];
	b.data[1]=a.data[0]*data[2]-a.data[2]*data[0];
  b.data[2]=a.data[1]*data[0]-a.data[0]*data[1];
	return b; };
template <class type> type matrix<type>::operator|(matrix<type>& a) {
	type sum=0;
	for (int c1=0;c1<MIN(numel,a.numel);c1++) sum+=data[c1]*a.data[c1];
	return sum; };
template <class type> matrix<type> matrix<type>::operator> (type a) {
	matrix<type> b(size[0],size[1],"");
	for (int c1=0;c1<numel;c1++) b.data[c1]=(data[c1]>a);
	return b; };
//preserving operators-------------------------------------------------------------------------
template <class type> void matrix<type>::operator+=(matrix<type>& a) {
	for (int c1=0;c1<MIN(numel,a.numel);c1++) data[c1]+=a.data[c1];};
template <class type> void matrix<type>::operator-=(matrix<type>& a) {
	for (int c1=0;c1<MIN(numel,a.numel);c1++) data[c1]-=a.data[c1];};
template <class type> void matrix<type>::operator*=(matrix<type>& a) {
	int c1,c2,c3; //NOTE: data!=a.data && size=a.size
	type *datacpy=new type[numel];
	memcpy(datacpy,data,sizeof(type)*numel);
	for (c1=0;c1<size[0];c1++)
		for (c2=0;c2<a.size[1];c2++) {
			data[c1+c2*size[0]]=0;
			for (c3=0;c3<size[1];c3++)
				data[c1+c2*size[0]]+=datacpy[c1+c3*size[0]]*a.data[c2*a.size[0]+c3];
		}
	delete[] datacpy;
};
template <class type> void matrix<type>::operator%=(matrix<type>& a) {
	for (int c1=0;c1<MIN(numel,a.numel);c1++) data[c1]*=a.data[c1];};	
template <class type> void matrix<type>::operator+=(type a) {
	for (int c1=0;c1<numel;c1++) data[c1]+=a;};
template <class type> void matrix<type>::operator-=(type a) {
	for (int c1=0;c1<numel;c1++) data[c1]-=a;};
template <class type> void matrix<type>::operator*=(type a) {
	for (int c1=0;c1<numel;c1++) data[c1]*=a;};
template <class type> void matrix<type>::operator/=(type a) {
	for (int c1=0;c1<numel;c1++) data[c1]=a/data[c1];};
template <class type> bool matrix<type>::operator==(array<type>& a) {
	if (numel!=a.numel) return false;
	for (int c1=0;c1<numel;c1++) 
		if (data[c1]!=a.data[c1]) return false;
	return true;
};
template <class type> void matrix<type>::operator&=(matrix<type>& a) {	
	type t0,t1;
	t0=a.data[2]*data[1]-a.data[1]*data[2];
	t1=a.data[0]*data[2]-a.data[2]*data[0];
  data[2]=a.data[1]*data[0]-a.data[0]*data[1];	
  data[0]=t0;
  data[1]=t1;
};

template <class type> type matrix<type>::glDet(void) {
	return -data[2]*data[5]*data[8]
				 +data[1]*data[6]*data[8]
				 +data[2]*data[4]*data[9]
				 -data[0]*data[6]*data[9]
				 -data[1]*data[4]*data[10]
				 +data[0]*data[5]*data[10];
};


//experiment:	
//template <class type> void matrix<type>::operator[](const int &a) {
//	printf("herddddddddddddddddddddddddddddde\n");
//	//return &data[a];
//};	
/*template <class type> type matrix<type>::operator[](const int& d0,const int& d1) {
	return data[d0+size[0]*d1];
};*/		

//functions-----------------------------------------------------------------
matrix<float> glRotation(float *a) {//Rx.Ry.Rz
	matrix<float> rot(4,4,"");
	float sina=sin(a[0]),cosa=cos(a[0]),
				sinb=sin(a[1]),cosb=cos(a[1]),
				sinc=sin(a[2]),cosc=cos(a[2]);
	rot.data[0]= cosb*cosc;									rot.data[4]=-cosb*sinc;									rot.data[ 8]= sinb;
  rot.data[1]= cosc*sina*sinb+cosa*sinc;  rot.data[5]= cosa*cosc-sina*sinb*sinc;	rot.data[ 9]=-cosb*sina;
  rot.data[2]=-cosa*cosc*sinb+sina*sinc;	rot.data[6]= cosc*sina+cosa*sinb*sinc;	rot.data[10]= cosa*cosb;    
	rot.data[15]=1;
  return rot;
};

matrix<float> glTranslate(float a,float b,float c) {//who did this
	matrix<float> tra(4,4,"eye");
	tra.data[12]=a;
	tra.data[13]=b;
	tra.data[14]=c;
	return tra;
};
matrix<float> glTranslate(float *a) {
	matrix<float> tra(4,4,"eye"); memcpy(&tra.data[12],a,12);
	return tra;
};

matrix<float> glScale(float a,float b,float c) {
	matrix<float> tra(4,4,"eye");
	tra.data[ 0]=a;
	tra.data[ 5]=b;
	tra.data[10]=c;
	return tra;
};


matrix<float> glIntegral(float dt,float ac,float my,float la) {
	matrix<float> x(3,3,"");
	x.data[3]=  dt*ac;
	x.data[4]=1-dt*my;
	x.data[6]=  dt*dt;
	x.data[7]=dt*(1-dt*my);
	x.data[8]=1-dt*la;
	return x;
}

void glAverageNormals(array<float> *v,array<unsigned int> *t,bool annul) {
	int c1,c2,c3;
	if (annul) for (c1=0;c1<v->numel;c1+=10) v->data[c1+4]=v->data[c1+5]=v->data[c1+6]=0;
	matrix<float> nrm(3,t->size[1],""),v0(3,1,""),v1(3,1,""),v2(3,1,"");	
	for (c1=0;c1<t->numel;c1+=3) {
		memcpy(v0.data,&v->data[10*t->data[c1  ]+7],12);
		memcpy(v1.data,&v->data[10*t->data[c1+1]+7],12);
		memcpy(v2.data,&v->data[10*t->data[c1+2]+7],12);
		v1-=v0;
		v2-=v0;
		v1&=v2;
		v1.normalize();
		memcpy(&nrm.data[c1],v1.data,12);
	}
	for (c1=0;c1<t->numel;c1+=3) //for all triangles T
		for (c2=0;c2<3;c2++)       //for the 3 vertices of t
			for (c3=0;c3<3;c3++)     //for the xyz coordinates
				v->data[10*t->data[c1+c2]+4+c3]+=nrm.data[c1+c3];
	for (c1=0;c1<v->numel;c1+=10) {		
		memcpy(v0.data,&v->data[c1+4],12);
		v0.normalize();
		memcpy(&v->data[c1+4],v0.data,12);
	}	
}

//tridiagonal solver-----------------------------------------------------
template <class type> class tridiag : public matrix<type> {
	public:
		tridiag<type>(void) {};
		matrix<type> solve(matrix<type> *b);			//solve LGS
	matrix<type> *l,*d,*r;
};

template <class type> matrix<type> tridiag<type>::solve(matrix<type> *b) {
	matrix<type> x(b->size[0],b->size[1],"");
	matrix<type> g(b->size[0],b->size[1],"");
	type beta=1/d->data[0];
	x.data[0]=b->data[0]*beta;
	for (int c1=1;c1<b->numel;c1++) {
		g.data[c1]=r->data[c1-1]*beta;
		x.data[c1]=(b->data[c1]-l->data[c1-1]*x.data[c1-1])*(beta=1/(d->data[c1]-l->data[c1-1]*g.data[c1]));
	}
	for (		 c1=b->numel-1;c1>0;c1--)
		x.data[c1-1]=x.data[c1-1]-g.data[c1]*x.data[c1];
	return x;
};

#endif